/* **********************************************************
 * Copyright (C) 1998-2000 VMware, Inc.
 * All Rights Reserved
 * $Id: vmx86.h,v 1.14.4.1 2002/05/20 21:12:10 hpreg Exp $
 * **********************************************************/

/*
 *  vmx86.h - Platform independent data and interface for supporting 
 *            the vmx86 device driver. 
 */

#ifndef VMX86_H
#define VMX86_H

#define INCLUDE_ALLOW_MODULE
#include "includeCheck.h"


/*
 * The ifdefs we use in the VMDriver structure are not really what I call
 * "platform independent"... --hpreg
 */
#ifdef __linux__
#   include "linux-version.h"
#endif

#include "machine.h"
#include "modulecall.h"
#include "vm_types.h"
#include "initblock.h"
#include "memmgmt.h"
#include "wslimits.h"


/*
 * VMDriver - the main data structure for the driver side of a 
 *            virtual machine.
 */ 

#define NUM_PASSTHROUGH_IRQS 10

enum vmEmulations { VME_UNKNOWN, VME_V2, VME_GSX1, VME_V3, VME_V32, VME_GSX2, VME_GSX25 };

typedef struct VMDriver {
   int                 count;        /* reference count */
   int                 id;           /* unique VM ID */
   struct VMDriver    *nextDriver;   /* Next on list of all VMDrivers */
   uintptr_t           processID;    /* User level process handle. */
   void               *uniqHandle;   /* Unique handle of driver */

   int                 logFD;        /* File descriptor for log messages */
   Vcpuid              numVCPUs;     /* Number of vcpus in VM. */
   /* Pointers to the crossover pages shared with the monitor. */
   void               *crosspage[MAX_INITBLOCK_CPUS];
   struct page        *crosspagePage;
#ifdef WINNT_DDK
   void               *crossPageMDL;
#endif
   volatile uint32   (*hostAPIC)[4]; /* kseg pointer to host APIC */
   unsigned int        hostAPIC_maxlvt;

   void               *memtracker;   /* Memory tracker pointer */
   struct PhysTracker *physTracker;
   int                 releaseCount;
   Bool                checkFuncFailed;
   struct PerfCounter *perfCounter;
   VMX86Stats	       stats;
   VMMemMgmtInfo       memMgmtInfo;
   WslimitsInfo        wslimitsInfo;

#ifdef SUPPORT_PASSTHROUGH
   /* When Passthrough IRQs fire, we buffer them here until queried */
   int                 numPendingPassthroughIRQs;
   unsigned char       pendingPassthroughIRQs[NUM_PASSTHROUGH_IRQS];
#endif
   uint32	       monContextVA;
   int32	       port;
   char		       name[VM_NAME_LENGTH];
   char		       userName[VM_USER_NAME_LENGTH];
   VMInfoState	       state;
   enum vmEmulations   vmVersion;
} VMDriver;


extern VMDriver *Vmx86_CreateVM(void *uniqHandle, uintptr_t processId, 
				WslimitsInfo *minfo, enum vmEmulations vmVersion);
extern VMDriver *Vmx86_BindVM(int id);
extern int Vmx86_ReleaseVM(VMDriver *vm);
extern int Vmx86_InitVM(VMDriver *vm, InitBlock *initParams);
extern int Vmx86_LateInitVM(VMDriver *vm);
extern int Vmx86_RunVM_GSX25(VMDriver *vm, Vcpuid vcpuid);
extern int Vmx86_RunVM_GSX2(VMDriver *vm, Vcpuid vcpuid);
extern int Vmx86_RunVM_V3(VMDriver *vm, Vcpuid vcpuid);
extern int Vmx86_RunVM_V32(VMDriver *vm, Vcpuid vcpuid);
extern void Vmx86_SetStartTime(VmTimeStart *st);
extern uint32 Vmx86_GetMHzEstimate(VmTimeStart *st);
extern VMDriver *Vmx86_GetVMforProcess(uintptr_t processId);
extern MPN Vmx86_LockPage(VMDriver *vm, void *addr, Bool userPage);
extern int Vmx86_UnlockPage(VMDriver *vm, void *addr, Bool userPage);
extern int32 Vmx86_GetNumVMs(void);
extern int32 Vmx86_GetTotalMemUsage(void);
extern int32 Vmx86_GetLockedPagesLimit(void);
extern Bool Vmx86_SetLockedPagesLimit(uintptr_t limit);
extern void Vmx86_SetLockedPageLimitMonitorStatus(Bool status);
extern uintptr_t Vmx86_ChangeLockedPagesLimit(uintptr_t limit);
extern Bool Vmx86_GetMemInfo(VMDriver *curVM,
                             int32 curVMOnly,
                             VMGetMemInfoArgs *args,
                             int outArgsLength);
extern Bool Vmx86_GetVMListCopy(VMGetVMListResult *list);
extern void Vmx86_SetVMInfo(VMDriver *curVM, VMSetVMInfoArgs *info);
extern Bool Vmx86_GetVMInfo(uint32 vmUID, VMGetVMInfoResult *info);
extern Bool Vmx86_GetMemInfoCopy(VMDriver *curVM,
				 VMGetMemInfoArgs *args);
extern void Vmx86_SetMemInfo(VMDriver *curVM, VMSetMemInfoArgs *args);
extern void Vmx86_Add2MonPageTable(VMDriver *vm, VPN vpn, MPN mpn,
				   Bool readOnly);
extern Bool Vmx86_PAEEnabled(void);

#endif 
