/* **********************************************************
 * Copyright (C) 1998-2001 VMware, Inc.
 * All Rights Reserved
 * $Id: vm_basic_types.h,v 1.24.2.1 2002/05/25 02:38:20 leb Exp $
 * **********************************************************/

/*
 *
 * vm_basic_types.h -- 
 *
 *    basic data types.
 */

 
#ifndef _VM_BASIC_TYPES_H_
#define _VM_BASIC_TYPES_H_

#define INCLUDE_ALLOW_USERLEVEL
#define INCLUDE_ALLOW_MONITOR
#define INCLUDE_ALLOW_MODULE
#define INCLUDE_ALLOW_VMKERNEL
#include "includeCheck.h"

/* STRICT ANSI means the Xserver build and X defines Bool differently. */
#ifndef __STRICT_ANSI__
typedef char           Bool;
#endif

#ifndef FALSE
#define FALSE          0
#endif

#ifndef TRUE
#define TRUE           1
#endif

/*
 * Macros __i386__ and __ia64 are intrinsically defined by GCC
 */
#ifdef __i386__
#define VM_I386
#endif

#ifdef __ia64__
#define VM_IA64
#endif

#ifdef _WIN32
/* safe assumption for a while */
#define VM_I386
#endif

#if defined VM_I386 && defined VM_IA64
#error "Only one CPU platform is allowed."
#endif

#if !defined VM_I386 && !defined VM_IA64
#error "One CPU platform has to be defined."
#endif

#ifdef _MSC_VER
typedef unsigned __int64 uint64;
typedef signed __int64 int64;

#pragma warning (3 :4505) // unreferenced local function
#pragma warning (disable :4018) // signed/unsigned mismatch
#pragma warning (disable :4761) // integral size mismatch in argument; conversion supplied
#pragma warning (disable :4305) // truncation from 'const int' to 'short'
#pragma warning (disable :4244) // conversion from 'unsigned short' to 'unsigned char'
#if !defined VMX86_DEVEL // XXX until we clean up all the code -- edward
#pragma warning (disable :4133) // incompatible types - from 'struct VM *' to 'int *'
#pragma warning (disable :4047) // differs in levels of indirection
#endif
#pragma warning (disable :4146) // unary minus operator applied to unsigned type, result still unsigned
#pragma warning (disable :4142) // benign redefinition of type

#elif __GNUC__
/* The Xserver source compiles with -ansi -pendantic */
#ifndef __STRICT_ANSI__
#ifdef VM_IA64
typedef unsigned long uint64;
typedef long int64;
#else
typedef unsigned long long uint64;
typedef long long int64;
#endif
#endif
#else
#error - Need compiler define for int64/uint64
#endif

typedef unsigned int       uint32;
typedef unsigned short     uint16;
typedef unsigned char      uint8;

typedef int       int32;
typedef short     int16;
typedef char      int8;

/*
 * FreeBSD (for the tools build) unconditionally defines these in sys/inttypes.h
 * so don't redefine them if this file has already been included. [greg]
 */
#if !defined(__FreeBSD__) || !defined(_SYS_INTTYPES_H_)
#   if !defined(__intptr_t_defined) && !defined(intptr_t) 
#      define __intptr_t_defined
#      define intptr_t  intptr_t
#      ifdef VM_I386
          typedef int32     intptr_t;
#      endif

#      ifdef VM_IA64
          typedef int64     intptr_t;
#      endif
#   endif

#   ifndef _STDINT_H
#      ifdef VM_I386
          typedef uint32    uintptr_t;
#      endif

#      ifdef VM_IA64
          typedef uint64    uintptr_t;
#      endif
#   endif 
#endif

/*
 * Printf format for 64-bit number.  Use it like this:
 *    printf("%"FMT64"d\n", big);
 */

#ifdef _MSC_VER
#define FMT64   "I64"
#elif __GNUC__
#define FMT64   "L"
#else
#error - Need compiler define for FMT64
#endif

typedef uintptr_t VA;
typedef uintptr_t VPN;

typedef uintptr_t PA;
typedef uintptr_t PPN;

typedef uint64     MA;
typedef uintptr_t MPN;

#define INVALID_PPN ((PPN)-1)
#define INVALID_MPN ((MPN)-1)
#define INVALID_LPN ((LPN)-1)
#define INVALID_VPN ((VPN)-1)

#define EXTERN        extern
#define CONST         const

#ifdef _MSC_VER
#ifndef INLINE
#define INLINE        __inline
#endif
#else
#ifndef INLINE
#define INLINE        inline
#endif
#endif


#if defined(WIN32) && !defined(VMX86_NO_THREADS)
#define THREADSPECIFIC _declspec(thread) 
#else
#define THREADSPECIFIC
#endif

/* 
 * Like "INLINE" but use this token to mark functions that are inline
 * because they have only a single call site. In other words, if a second
 * call site is introduced, the "INLINE_SINGLE_CALLER" token should be 
 * removed. 
 */
#define INLINE_SINGLE_CALLER INLINE

/*
 * Attributes placed on function declarations to tell the compiler
 * that the function never returns.
 */
#ifdef _MSC_VER
#define NORETURN_DECL(_fndecl)    __declspec(noreturn) _fndecl
#elif __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5)
#define NORETURN_DECL(_fndecl)    _fndecl __attribute__((__noreturn__))
#else
#define NORETURN_DECL(_fndecl)    _fndecl
#endif


/*
 * GCC's argument checking for printf-like functions
 * This is conditional until we have replaced all `"%x", void *'
 * with `"0x%08x", (uint32) void *'. Note that %p prints different things
 * on different platforms.  Argument checking is enabled for the
 * vmkernel, which has already been cleansed.
 *
 * fmtPos is the position of the format string argument, beginning at 1
 * varPos is the position of the variable argument, beginning at 1
 */
#if defined(__GNUC__)
# define PRINTF_DECL(fmtPos, varPos) __attribute__((__format__(__printf__, fmtPos, varPos)))
#else
# define PRINTF_DECL(fmtPos, varPos)
#endif

/*
 * Used to silence compiler warnings that get generated when the
 * compiler thinks that a function returns when it is marked noreturn.
 */
#define INFINITE_LOOP()           do { } while (1)

/*
 * Global error codes. Currently used internally, but may be exported
 * to customers one day, like VM_E_XXX in vmcontrol_constants.h
 */
typedef enum VMwareStatus {
   VMWARE_STATUS_SUCCESS,  // success
   VMWARE_STATUS_ERROR,    // generic error
   VMWARE_STATUS_NOMEM,    // generic memory allocation error
   VMWARE_STATUS_INSUFFICIENT_RESOURCES, // internal or system resource limit exceeded
   VMWARE_STATUS_INVALID_ARGS  // invalid arguments
} VMwareStatus;

#define VMWARE_SUCCESS(s) ((s) == VMWARE_STATUS_SUCCESS)

#endif
